﻿/*

The MIT License

Copyright (c) 2005-2008 Elvis Mehmedović

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

http://emehmedovic.com/

*/

import xm.corpus.resizableComponent;
import flash.display.BitmapData;

class xm.components.ImageContainerComponent extends resizableComponent {
	
	[IconFile("ImageContainerComponent.png")]
	
	[Event("loadStart")]
	[Event("loadComplete")]
	[Event("loadInit")]
	[Event("loadProgress")]
	
	static var rmFitIntoAR:String = "fit into component area / mind aspect ratio";
	static var rmFitInto:String = "fit into component area";
	static var rmFillOut:String = "fill out component area";
	static var rmCenter:String = "center in component area";
	static var rmCenterOrFitInto:String = "center in component area / shrink if necessary";
	static var rmComponentToImage:String = "resize component to image size";

	static var ltQueue:String = "queue";
	static var ltParallel:String = "parallel";
	
	static var globalPreventImageCaching:Boolean;
	
	private static var imageCounter:Number = 0;
	
	private var _iid:Number;
	private var _initializing:Boolean;
	private var _mask:MovieClip;
	private var loader:MovieClipLoader;
	private var imageQueue:Array;
	
	private var __applyMask:Boolean;
	private var __resizeMode:String = rmFitInto;
	private var __url:String;
	
	var _container:MovieClip;
	var anyLoading:Boolean;
	var anyShowing:Boolean;
	
	[Inspectable()]
	var smoothing:Boolean = true;

	[Inspectable()]
	var smoothTransparency:Boolean = false;
	
	[Inspectable()]
	var retryDelay:Number = 0.5;
	
	[Inspectable()]
	var preventImageCaching:Boolean = false;
	
	[Inspectable(defaultValue="queue", enumeration="queue,parallel")]
	var loadingType:String = ltQueue;
	
	[Inspectable(defaultValue="fit into component area / mind aspect ratio", enumeration="fit into component area / mind aspect ratio,fit into component area,fill out component area,center in component area,top left in component area,center in component area / shrink if necessary,resize component to image size")]
	function get resizeMode():String {
		return __resizeMode;
	}
	
	[Inspectable()]
	var supraQueueing:Boolean = false;
	
	[Inspectable()]
	var supraQueueParentTarget:String = "_parent";
	
	[Inspectable()]
	var supraQueueWidth:Number = 1;
	
	function set resizeMode(p_mode:String) {
		
		__resizeMode = p_mode;
		
		if (_currentframe == 1) {
			return;
		}
		
		refresh();
		
	}
	
	function setSize(p_width:Number, p_height:Number):Void {
		
		super.setSize(p_width, p_height);
		
		if (resizeMode != rmComponentToImage) {
			refresh();
		}
		
	}
		
	[Inspectable(defaultValue=true)]
	function get applyMask():Boolean {
		return __applyMask;
	}
	
	function set applyMask(p_mode:Boolean) {
		
		if (p_mode == applyMask && !_initializing) {
			return;
		}
		
		__applyMask = p_mode;
		
		if (_currentframe == 1) {
			return;
		}
		
		if (applyMask) {
			attachMovie("ImageContainer Component: Mask", "_mask", getNextHighestDepth());
			Aligner.register(_mask, { hStretch: true, vStretch: true });
			_container.setMask(_mask);
		}
		else {
			_mask.removeMovieClip();
			_container.setMask(null);
		}
		
	}

	[Inspectable()]
	function get url():String {
		return __url;
	}

	function set url(p_val:String) {

		if (((preventImageCaching || globalPreventImageCaching) && (url.substr(0, 7) == "http://" || !Boolean(_global.MMSave))) && p_val != undefined && p_val != "" && !_initializing) {
			if (p_val.indexOf("?") >= 0) {
				p_val += "&rnd="+getTimer()+"_"+random(999999999);
			}
			else {
				p_val += "?rnd="+getTimer()+"_"+random(999999999);
			}
		}
		
		if ((p_val == url && !_initializing) || p_val == undefined || p_val == "") {
			return;
		}

		__url = p_val;
		
		if (_currentframe == 1) {
			return;
		}

		addImageToSupraQueue(url);

	}
	
	private function addImageToQueue(p_url:String):Void {
		
		var c:MovieClip = _container.createEmptyMovieClip(String(++imageCounter), _container.getNextHighestDepth());
		c = c.createEmptyMovieClip(String(imageCounter), c.getNextHighestDepth());
		imageQueue.push({mc: c, url: p_url});
		
		if (loadingType == ltParallel) {
			clearNotLoadedImagesBelow(c);
		}
		
		if (!(loadingType == ltQueue && anyLoading)) {
			loadNextImage();
		}
		
	}
	
	private function loadNextImage() {
		
		for (var i:Number = 0; i < imageQueue.length; i++) {
			var o:Object = imageQueue[i];
			if (!o.started) {
				o.started = true;
				anyLoading = true;
				loader.loadClip(o.url, o.mc);
				dispatchEvent({type: "loadStart", image: o.mc});
				return;
			}
		}
		
		anyLoading = false;
		
	}

	function clear(Void):Void {
		
		while (imageQueue.length) {
			
			var o:Object = imageQueue[0];
			
			o.mc.removeMovieClip();
			imageQueue.shift();
			
		}

		delete __url;
		anyShowing = false;
		
		if (anyLoading) {
			dispatchEvent({type: "loadComplete"});
		}
		
		loadNextImage();
		
	}
	
	function clearImagesBelow(p_cont:MovieClip):Void {
		
		while (imageQueue.length) {
			
			var o:Object = imageQueue[0];
			
			if (!o.started || o.mc == p_cont) {
				break;
			}
			
			o.mc.removeMovieClip();
			imageQueue.shift();
			
		}
		
	}
	
	function clearNotLoadedImagesBelow(p_cont:MovieClip):Void {
		
		var i:Number = 0;
		
		while (i < imageQueue.length) {
			
			var o:Object = imageQueue[i];
			
			if (o.mc == p_cont) {
				break;
			}
			
			if (!o.mc.imageLoaded) {
				o.mc.removeMovieClip();
				imageQueue.splice(i, 1);
			}
			else {
				i++;
			}
			
		}
		
	}

	private function findInQueue(p_mc:MovieClip):Object {
		
		for (var i:Number = 0; i < imageQueue.length; i++) {
			if (imageQueue[i].mc == p_mc) {
				return imageQueue[i];
			}
		}
		
	}
	
	function getImageBelow(p_mc:MovieClip):MovieClip {
		
		var mc:MovieClip;
		
		for (var i:Number = 0; i < imageQueue.length; i++) {
			if (imageQueue[i].mc == p_mc) {
				return mc;
			}
			if (imageQueue[i].mc.imageLoaded) {
				mc = imageQueue[i].mc;
			}
		}
		
	}
	
	private function _retryTimeout(Void):Void {
		
		clearInterval(_iid);
		delete _iid;
		
		loadNextImage();
		
	}
	
	private function onLoadError(p_mc:MovieClip):Void {
		
		var o:Object = findInQueue(p_mc);
		
		if (!o) {
			return;
		}
		
		if (p_mc == imageQueue[imageQueue.length-1].mc) {
			o.started = false;
			clearInterval(_iid);
			_iid = setInterval(this, "_retryTimeout", retryDelay * 1000);
		}
		else {
			loadNextImage();
		}
		
	}

	private function onLoadComplete(p_mc:MovieClip):Void {
		
		var o:Object = findInQueue(p_mc);
		
		if (!o) {
			return;
		}
		
		p_mc.url = findInQueue(p_mc).url;
		
		p_mc.imageLoaded = true;
		clearNotLoadedImagesBelow(p_mc);
		
		dispatchEvent({type: "loadComplete", image: p_mc});
		
	}

	private function onLoadInit(p_mc:MovieClip):Void {
		
		var o:Object = findInQueue(p_mc);
		
		if (!o) {
			return;
		}
		
		anyShowing = true;

		if (smoothing) {
			
			var url = p_mc.url;
			
			var bitmap:BitmapData = new BitmapData(p_mc._width, p_mc._height, smoothTransparency, 0);
			bitmap.draw(p_mc);
			
			var c:MovieClip = p_mc._parent;
			
			p_mc = c.createEmptyMovieClip(p_mc._name, p_mc.getDepth());
			p_mc.attachBitmap(bitmap, 0, "never", true);

			p_mc.url = url;
			p_mc.imageLoaded = true;
			
		}
		
		p_mc.imageLoaded = true;
		p_mc.imageReady = true;
		
	
		if (!p_mc.preferredWidth) {
			p_mc.preferredWidth = p_mc._width;
			p_mc.preferredHeight = p_mc._height;
		}
		
		refreshSingle(p_mc);
		
		dispatchEvent({type: "loadInit", image: p_mc});
		
		loadNextImage();
		
		if (supraQueueing) {
			supraQueued--;
			processSupraQueue();
		}
		
	}
	
	function refresh(Void):Void {
		
		for (var i:Number = 0; i < imageQueue.length; i++) {
			if (imageQueue[i].mc.imageReady) {
				refreshSingle(imageQueue[i].mc);
			}
		}

	}

	function refreshSingle(p_mc:MovieClip):Void {
		
		var rm:String = resizeMode;
		
		if (rm == rmCenterOrFitInto) {
			rm = (p_mc.preferredWidth > width || p_mc.preferredHeight > height) ? rmFitIntoAR : rmCenter;
		}
		
		if (rm == rmCenter) {
			
			p_mc._width = p_mc.preferredWidth;
			p_mc._height = p_mc.preferredHeight;
			p_mc._x = Math.ceil((width - p_mc.preferredWidth)/2);
			p_mc._y = Math.ceil((height - p_mc.preferredHeight)/2);
			
		}
		else if (rm == rmFitInto) {
			
			p_mc._width = width;
			p_mc._height = height;
			p_mc._x = 0;
			p_mc._y = 0;
			
		}
		else if (rm == rmComponentToImage) {
			
			p_mc._width = p_mc.preferredWidth;
			p_mc._height = p_mc.preferredHeight;
			p_mc._x = 0;
			p_mc._y = 0;
			setSize(p_mc.preferredWidth, p_mc.preferredHeight);
			
		}
		else {
			
			var cond:Boolean = width/height > p_mc.preferredWidth/p_mc.preferredHeight;
			if (rm == rmFillOut ? cond : !cond) {
				p_mc._width = width;
				p_mc._yscale = p_mc._xscale;
				p_mc._y = -Math.round((p_mc._height - height)/2);
				p_mc._x = 0;
			}
			else {
				p_mc._height = height;
				p_mc._xscale = p_mc._yscale;
				p_mc._x = -Math.round((p_mc._width - width)/2);
				p_mc._y = 0;
			}
			
		}
		
	}

	private function onLoadProgress(p_mc:MovieClip, p_bl:Number, p_bt:Number):Void {
		
		var o:Object = findInQueue(p_mc);
		
		if (!o) {
			return;
		}
		
		if (p_bt > 0) {
			dispatchEvent({type: "loadProgress", bl: p_bl, bt: p_bt, image: p_mc});
		}

	}
	
	function get supraQueueParent():MovieClip {
		var sqParent:MovieClip = evaluateTarget(supraQueueParentTarget);
		if (!sqParent.__supraQueue) {
			sqParent.__supraQueued = 0;
			sqParent.__supraQueue = new Array();
		}
		return sqParent;
	}
	
	function get supraQueue():Array {
		return supraQueueParent.__supraQueue;
	}
	
	function set supraQueued(p_val:Number) {
		supraQueueParent.__supraQueued = p_val;
	}
	
	function get supraQueued():Number {
		return supraQueueParent.__supraQueued;
	}

	private function addImageToSupraQueue(p_url:String):Void {

		if (supraQueueing) {
			supraQueue.push({o: this, url: p_url});
			processSupraQueue();
		}
		else {
			addImageToQueue(p_url);
		}
		
	}
	
	private function processSupraQueue(Void):Void {
		
		while (supraQueue.length && supraQueued < supraQueueWidth) {
			var queueItem:Object = supraQueue.shift();
			queueItem.o.addImageToQueue(queueItem.url);
			supraQueued++;
		}
		
	}

	function ImageContainerComponent() {
	}

	function init(Void):Void {

		super.init();
		nextFrame();

		imageQueue = new Array();

		loader = new MovieClipLoader();
		loader.addListener(this);
		
		_initializing = true;

		applyMask = applyMask;
		url = url;
		
		_initializing = false;

	}

}