﻿/*

The MIT License

Copyright (c) 2005-2008 Elvis Mehmedović

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

http://emehmedovic.com/

*/

import xm.components.GenericScrollbarClient;
import xm.util.miniTweener;

class xm.components.ScrollAreaComponent extends GenericScrollbarClient {
	
	[IconFile("ScrollAreaComponent.png")]
	
	[Inspectable()]
	var keyboardHStep:Number = 35;
	
	[Inspectable()]
	var keyboardVStep:Number = 35;
	
	[Inspectable()]
	var mouseWheelHStep:Number = 35;
	
	[Inspectable()]
	var mouseWheelVStep:Number = 35;
	
	[Inspectable(defaultValue=true)]
	var easing:Boolean;
	
	[Inspectable(defaultValue=0.5)]
	var easingDuration:Number;
	
	var autoUpdate:Boolean = true;
	var easingFunction:Function = miniTweener.easeOutExpo;

	var content:MovieClip;
	var mcLoader:MovieClipLoader;
	
	private var _m:MovieClip;
	
	private var __manualWidth:Number;
	private var __manualHeight:Number;
	
	private var $fix:Number;
	private var $sfix:Number;

	private var __easing:Boolean;
	private var __mouseWheel:String;
	private var __keyboard:String;
	private var __applyMask:Boolean = true;
	private var __symbol:String;
	private var __url:String;
	
	private var _t:Number;
	
	private var gotoX:Number = 0;
	private var gotoY:Number = 0;
	private var lcontent:MovieClip;
	
	[Inspectable()]
	function get url():String {
		return __url;
	}

	function set url(p_val:String) {
		
		__url = p_val;
		
		if (width == undefined) {
			return;
		}
		
		$sfix = $sfix > 1000 || !$sfix ? 1 : $sfix + 1;
		
		lcontent.removeMovieClip();
		lcontent = createEmptyMovieClip("lcontent"+$sfix, getNextHighestDepth());
		
		mcLoader.loadClip(p_val, lcontent);
		
	}
	
	function get manualWidth():Number {
		return __manualWidth;
	}

	function set manualWidth(p_val:Number) {
		
		__manualWidth = p_val;
		_onEnterFrame();

	}
	
	function get manualHeight():Number {
		return __manualHeight;
	}
	
	private function onLoadInit(Void):Void {
		
		onEnterFrame = _onEnterFrame;
		
		gotoX = 0;
		gotoY = 0;
		
		content.removeMovieClip();
		content = lcontent;
		delete lcontent;
		
		applyMask = applyMask;
		
		update();
		
	}
	
	private function _update(Void):Void {
		
		update();
		onEnterFrame = _onEnterFrame;
		
	}
	
	private function _onEnterFrame(Void):Void {
		
		if (autoUpdate && (oldContentWidth != contentWidth || oldContentHeight != contentHeight || oldWidth != width || oldHeight != height)) {
			if (scroll > maxscroll) {
				scroll = maxscroll;
			}
			if (xhscroll > xmaxhscroll) {
				xhscroll = xmaxhscroll;
			}
			update();
		}
		
	}
	
	[Inspectable(defaultValue="none", enumeration="none,auto,horizontalScroll,verticalScroll")]
	function get keyboard():String {
		return __keyboard;
	}

	function set keyboard(p_val:String) {
		
		Key.removeListener(this);
		
		if ((__keyboard = p_val) != "none") {
			Key.addListener(this);
		}

	}

	[Inspectable(defaultValue="auto", enumeration="none,auto,horizontalScroll,verticalScroll,diagonalScroll")]
	function get mouseWheel():String {
		return __mouseWheel;
	}

	function set mouseWheel(p_val:String) {
		
		Mouse.removeListener(this);
		
		if ((__mouseWheel = p_val) != "none") {
			Mouse.addListener(this);
		}

	}
	
	[Inspectable()]
	function get symbol():String {
		return __symbol;
	}

	function set symbol(p_val:String) {
		
		__symbol = p_val;
		
		if (width == undefined) {
			return;
		}
		
		onEnterFrame = _onEnterFrame;
		
		gotoX = 0;
		gotoY = 0;
		
		$fix = $fix > 1000 || !$fix ? 1 : $fix + 1;
		
		content.removeMovieClip();
		content = attachMovie(p_val, "content"+$fix, getNextHighestDepth(), {scrollArea: this});
		
		applyMask = applyMask;
		
		update();
		
	}
	
	private function onMouseWheel(p_delta:Number, t_scrollTarget:MovieClip):Void {
		
		if (_xmouse >= 0 && _xmouse < width && _ymouse >= 0 && _ymouse < height) {
			if (mouseWheel == "horizontalScroll") {
				xhscroll -= p_delta * mouseWheelHStep;
			}
			else if (mouseWheel == "verticalScroll") {
				scroll -= p_delta * mouseWheelVStep;
			}
			else if (mouseWheel == "diagonalScroll") {
				xhscroll -= p_delta * mouseWheelHStep;
				scroll -= p_delta * mouseWheelVStep;
			}
			else if (mouseWheel == "auto") {
				if (maxscroll == 1) {
					xhscroll -= p_delta * mouseWheelHStep;
				}
				else {
					scroll -= p_delta * mouseWheelVStep;
				}
			}
		}
		
	}
	
	private function onKeyDown(Void):Void {
		
		if (keyboard == "none" || eval(Selection.getFocus()).constructor == TextField && eval(Selection.getFocus()).type == "input") {
			return;
		}
		
		if (Key.getCode() == Key.UP || Key.getCode() == Key.LEFT) {
			if (keyboard == "horizontalScroll" || keyboard == "auto" && (maxscroll == 1 || Key.getCode() == Key.LEFT)) {
				xhscroll += -keyboardHStep;
			}
			else if (keyboard == "verticalScroll" || keyboard == "auto" && (xmaxhscroll == 1 || Key.getCode() == Key.UP)) {
				scroll += -keyboardVStep;
			}
		}
		else if (Key.getCode() == Key.DOWN || Key.getCode() == Key.RIGHT) {
			if (keyboard == "horizontalScroll" || keyboard == "auto" && (maxscroll == 1 || Key.getCode() == Key.RIGHT)) {
				xhscroll += keyboardHStep;
			}
			else if (keyboard == "verticalScroll" || keyboard == "auto" && (xmaxhscroll == 1 || Key.getCode() == Key.DOWN)) {
				scroll += keyboardVStep;
			}
		}
		else if (Key.getCode() == Key.PGUP) {
			if (keyboard == "horizontalScroll" || keyboard == "auto" && maxscroll == 1) {
				xhscroll += -(width - 25);
			}
			else {
				scroll += -(height - 25);
			}
		}
		else if (Key.getCode() == Key.PGDN) {
			if (keyboard == "horizontalScroll" || keyboard == "auto" && maxscroll == 1) {
				xhscroll += width - 25;
			}
			else {
				scroll += height - 25;
			}
		}
		else if (Key.getCode() == Key.HOME) {
			if (keyboard == "horizontalScroll" || keyboard == "auto" && maxscroll == 1) {
				xhscroll = 0;
			}
			else {
				scroll = 0;
			}
		}
		else if (Key.getCode() == Key.END) {
			if (keyboard == "horizontalScroll" || keyboard == "auto" && maxscroll == 1) {
				xhscroll = Infinity;
			}
			else {
				scroll = Infinity;
			}
		}
		
	}
	
  	[Inspectable(defaultValue=true)]
	function get applyMask():Boolean {
		return __applyMask;
	}

	function set applyMask(p_val:Boolean) {
		
		__applyMask = p_val;
		
		if (width == undefined) {
			return;
		}
		
		if (p_val) {
			gotoAndStop(3);
			Aligner.register(_m, { hStretch: true, vStretch: true });
			content.setMask(_m);
		}
		else {
			gotoAndStop(2);
			content.setMask(null);
		}

	}
	
	function ScrollAreaComponent() {
	}
	
	function init(Void):Void {
		
		super.init();
		
		mcLoader = new MovieClipLoader();
		mcLoader.addListener(this);
		
		symbol = symbol;
		url = url;
		
	}
	
	function resetContentPosition() {
		
		var wasEasing:Boolean = easing;
		
		easing = false;
		scroll = 1;
		xhscroll = 1;
		easing = wasEasing;
		
	}
	
	// Override GenericScrollbarClient
	
		function get contentX():Number {
			return gotoX;
		}
		
		function set contentX(p_val:Number):Void{
			gotoX = p_val;
			if (easing) {
				miniTweener.addTween(content, { _x: gotoX, time: easingDuration, transition: easingFunction, rounded: true });
			} else {
				content._x = p_val;
			}
		}
		
		function get contentY():Number {
			return gotoY;
		}
		
		function set contentY(p_val:Number):Void{
			gotoY = p_val;
			if (easing) {
				miniTweener.addTween(content, { _y: gotoY, time: easingDuration, transition: easingFunction, rounded: true });
			} else {
				content._y = p_val;
			}
		}
		
		function get contentHeight():Number {
			if (manualHeight != undefined) {
				return Math.ceil(manualHeight);
			}
			else {
				return Math.ceil(content._height);
			}
		}
		
		function get contentWidth():Number {
			if (manualWidth != undefined) {
				return Math.ceil(manualWidth);
			}
			else {
				return Math.ceil(content._width);
			}
		}

		function addListener(p_obj:Object):Void {
			super.addListener(p_obj);
			p_obj.vStep = 10;
		}
	
}