﻿/*

The MIT License

Copyright (c) 2007-2008 Elvis Mehmedović

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

http://emehmedovic.com/

*/

import xm.corpus.resizableComponent;
import xm.util.quickParsedXML;
import xm.components.FLVDisplayComponent;
import xm.components.FLVControllerComponent;
import xm.components.ImageContainerComponent;

class xm.components.ToobPlayerComponent extends resizableComponent {
	
	[IconFile("FLVDisplayComponent.png")]
	
	[Event("prePlay")]
	[Event("preStop")]
	[Event("showOrHideFullScreenButton")]
		
	[Event("enterFullScreen")]
	[Event("exitFullScreen")]
	
	static var rmFitIntoAR:String = "fit into component area / mind aspect ratio";
	static var rmFitInto:String = "fit into component area";
	static var rmFillOut:String = "fill out component area";
	static var rmCenter:String = "center in component area";
	static var rmCenterOrFitInto:String = "center in component area / shrink if necessary";
	static var rmComponentToVideo:String = "resize component to video size";
	
	private var __url:String;
	private var __previewImageUrl:String;
	private var __autoPlay:Boolean;
	private var __autoRepeat:Boolean;
	private var __videoResizeMode:String;
	private var __previewImageResizeMode:String;
	private var __mouseWheel:String;
	private var __volume:Number;
	private var __showFullScreenButton:Boolean;
	
	private var initialized:Boolean;
	private var youTubeProxyXML:quickParsedXML;
	private var youTubeId:String;
	
	var isAcquiringYouTubeUrl:Boolean = false;
	var preStatus:String;
	var isFullScreen: Boolean = false;
	
	var display:FLVDisplayComponent;
	var ctrl:FLVControllerComponent;
	var image:ImageContainerComponent;
	
	function ToobPlayerComponent() {
	}

	function init(Void):Void {
		
		super.init();
		
		tabChildren = false;
		
	}
	
	function setFullScreen(p_fs:Boolean):Void {
		if (isFullScreen == p_fs) {
			return;
		}
		isFullScreen = p_fs;
		if (isFullScreen) {
			dispatchEvent({type: "enterFullScreen"});
		}
		else {
			dispatchEvent({type: "exitFullScreen"});
		}
	}
	
	function lateInit(Void):Void {
		
		initialized = true;
		
		volume = volume;
		autoPlay = autoPlay;
		autoRepeat = autoRepeat;
		videoResizeMode = videoResizeMode;
		previewImageResizeMode = previewImageResizeMode;
		
		display.addEventListenerL("start", this, videoStart);
		display.addEventListenerL("playing", this, videoPlaying);
		display.addEventListenerL("closed", this, videoClosed);
		display.addEventListenerL("complete", this, videoComplete);
		
		previewImageUrl = previewImageUrl;
		url = url;
		
	}
	
	function prePlay(Void):Void {
		preStatus = "play";
		display.dispatchEvent({type: "prePlay"});
	}
	
	function preStop(Void):Void {
		preStatus = "stop";
		display.dispatchEvent({type: "preStop"});
	}
	
	[Inspectable(defaultValue=false)]
	var autoLoadYouTubePreviewImage:Boolean = false;

	[Inspectable(defaultValue=true)]
	var preferHigherQuality:Boolean = true;
	
	[Inspectable()]
	function get url():String {
		return __url;
	}

	[Inspectable()]
	function get previewImageUrl():String {
		return __previewImageUrl;
	}

	[Inspectable()]
	function get showFullScreenButton():Boolean {
		return __showFullScreenButton;
	}
	
	function set showFullScreenButton(p_val:Boolean) {
		__showFullScreenButton = p_val;
		dispatchEvent({type: "showOrHideFullScreenButton"});
	}
	
	function loadYouTubePreviewImage(Void):Void {
		previewImageUrl = youTubeProxy+"?image="+escape("http://img.youtube.com/vi/" + youTubeId + "/" + (preferHigherQuality ? "hq" : "") + "default.jpg");
	}

	// listener
	private function videoStart(Void):Void {
		this.display._visible = true
		this.image._visible = false;
	}
	
	// listener
	private function videoPlaying(Void):Void {
		this.display._visible = true
		this.image._visible = false;
	}
	
	// listener
	private function videoClosed(Void):Void {
		this.display._visible = false;
		this.image._visible = true;
	}

	// listener
	private function videoComplete(Void):Void {
		this.display._visible = false;
		this.image._visible = true;
	}
	
	function set url(p_val:String) {
		
		__url = p_val;
		
		if (url == "" || url == undefined || !initialized) {
			return;
		}

		display.stop();
		youTubeId = extractYouTubeId(url);
		
		if (youTubeId) {
			
			display.preferredWidth = 320;
			display.preferredHeight = 240;
			
			if (autoLoadYouTubePreviewImage) {
				loadYouTubePreviewImage();
			}
			
			isAcquiringYouTubeUrl = true;
			
			delete youTubeProxyXML.onXMLLoad;
			delete youTubeProxyXML.onError;
			youTubeProxyXML = new quickParsedXML(youTubeProxy+"?id="+escape(youTubeId) + "&hq="+Number(preferHigherQuality));
			youTubeProxyXML.retry = false;
			youTubeProxyXML["toobPlayerRef"] = this;

			youTubeProxyXML.onError = function() {
				this.toobPlayerRef.preStop();
				this.toobPlayerRef.isAcquiringYouTubeUrl = false;
				this.toobPlayerRef.display.dispatchEvent({type: "errorAcquiringYouTubeUrl"});
			}
			
			youTubeProxyXML.onXMLLoad = function() {
				
				var videoData:Object = this.defaultParse()[0];
				
				if (!videoData.video_id) {
					this.onError();
				}
				else {
					this.toobPlayerRef.isAcquiringYouTubeUrl = false;
					delete this.toobPlayerRef.preStatus;
					this.toobPlayerRef.display.url = videoData.video_id;
					if (!this.toobPlayerRef.autoPlay && this.toobPlayerRef.ctrl.playpause._currentFrame == 2) {
						this.toobPlayerRef.display.play();
					}
					this.toobPlayerRef.display.dispatchEvent({type: "acquiredYouTubeUrl"});
				}
				
			}
			
			display.dispatchEvent({type: "acquiringYouTubeUrl"});
			
			if (autoPlay) {
				// display pause button before the video is actually playing
				prePlay();
			}
			
		}
		else {
			delete youTubeProxyXML.onXMLLoad;
			delete youTubeProxyXML.onError;
			isAcquiringYouTubeUrl = false;
			display.url = url;
		}

	}
	
	function set previewImageUrl(p_val:String) {
		
		__previewImageUrl = p_val;
		
		if (previewImageUrl == "" || previewImageUrl == undefined || !initialized) {
			return;
		}
		
		image.url = previewImageUrl;

	}	
	
	[Inspectable(defaultValue="http://emehmedovic.com/xmca/toobplayer/fl_youTubeProxy.php")]
	var youTubeProxy:String;
	
	[Inspectable(defaultValue=true)]
	function get autoPlay():Boolean {
		return __autoPlay;
	}
	
	function set autoPlay(p_val:Boolean) {
		__autoPlay = p_val;
		display.autoPlay = autoPlay;
	}

	[Inspectable(defaultValue=false)]
	function get autoRepeat():Boolean {
		return __autoRepeat;
	}
	
	function set autoRepeat(p_val:Boolean) {
		__autoRepeat = p_val;
		display.autoRepeat = autoRepeat;
	}
	
	[Inspectable(defaultValue=50)]
	function get volume():Number {
		return __volume;
	}
	
	function set volume(p_val:Number) {
		__volume = p_val;
		display.volume = volume;
	}
	
	[Inspectable(defaultValue="fit into component area / mind aspect ratio", enumeration="fit into component area / mind aspect ratio,fit into component area,fill out component area,center in component area,center in component area / shrink if necessary,resize component to video size")]
	function get videoResizeMode():String {
		return __videoResizeMode;
	}
	
	function set videoResizeMode(p_val:String) {
		__videoResizeMode = p_val;
		display.resizeMode = videoResizeMode;
	}
	
	[Inspectable(defaultValue="fill out component area", enumeration="fit into component area / mind aspect ratio,fit into component area,fill out component area,center in component area,center in component area / shrink if necessary,resize component to video size")]
	function get previewImageResizeMode():String {
		return __previewImageResizeMode;
	}
	
	function set previewImageResizeMode(p_val:String) {
		__previewImageResizeMode = p_val;
		image.resizeMode = previewImageResizeMode;
	}
	
	function checkYouTubeId(p_id:String, p_handler:Function):Void {
		
		var checkYouTubeIdXML = new quickParsedXML(youTubeProxy+"?id="+escape(p_id));
		checkYouTubeIdXML.retry = false;
		checkYouTubeIdXML.handler = p_handler;

		checkYouTubeIdXML.onError = function() {
			this.handler(false);
		}
		
		checkYouTubeIdXML.onXMLLoad = function() {
			
			var videoData:Object = this.defaultParse()[0];
			
			if (!videoData.video_id) {
				this.handler(false, p_id);
			}
			else {
				this.handler(true, p_id);
			}
			
		}
			
	}
	
	static function extractYouTubeId(p_url:String):String {
		var i:Number;
		if ((i = p_url.indexOf("youtube.com/watch?v=")) != -1) {
			var s:String = p_url.substr(i+20);
			i = s.indexOf("&");
			if (i < 0) i = undefined;
			return s.substr(0, i);
		}
		else if ((i = p_url.indexOf("youtube.com/v/")) != -1) {
			var s:String = p_url.substr(i+14);
			var i1:Number = s.indexOf("&");
			var i2:Number = s.indexOf("'");
			var i3:Number = s.indexOf("\"");
			if (i1 < 0) i1 = Infinity;
			if (i2 < 0) i2 = Infinity;
			if (i3 < 0) i3 = Infinity;
			i = Math.min(i3, Math.min(i1, i2));
			if (i == Infinity) i = undefined;
			return s.substr(0, i);
		}
		else if (p_url.indexOf(".") == -1) {
			return p_url;
		}
	}
	
	var mouseWheelVolumeStep:Number = 5;
	var mouseWheelSeekStep:Number = 5;
	
	[Inspectable(defaultValue="volume", enumeration="none,seek,volume")]
	 function get mouseWheel():String {
		return __mouseWheel;
	}

	function set mouseWheel(p_val:String) {

		Mouse.removeListener(this);

		if ((__mouseWheel=p_val) != "none") {
			Mouse.addListener(this);
		}

	}
	
	private var _wheelSeekIId:Number;
	private var _wheelSeekTime:Number;
	private var _wheelSeekWasPlaying:Boolean;
	
	private function _wheelSeekStop(p_toob:ToobPlayerComponent) {
		clearInterval(p_toob._wheelSeekIId);

		delete p_toob._wheelSeekIId;
		if (p_toob._wheelSeekWasPlaying) {
			p_toob.display.disablePlayingEvents = true;
			p_toob.display.time = p_toob.display.time;
			p_toob.display.pause(false);
			p_toob.display.disablePlayingEvents = false;
			p_toob.display.time = p_toob.display.time;
		}
	}

	private function onMouseWheel(p_delta:Number, t_scrollTarget:MovieClip):Void {
		
		if (_xmouse >= 0 && _xmouse < width && _ymouse >= 0 && _ymouse < height) {
			if (mouseWheel == "volume") {
				display.volume = Math.max(0, Math.min(100, display.volume - p_delta * mouseWheelVolumeStep));
			}
			else if (mouseWheel == "seek") {
				if (_wheelSeekIId == undefined) {
					_wheelSeekIId = setInterval(_wheelSeekStop, 350, this);
					_wheelSeekWasPlaying = display.playing;
					_wheelSeekTime = display.time;

					display.disablePlayingEvents = true;
					display.pause(true);
					display.disablePlayingEvents = false;
				}
				else {
					clearInterval(_wheelSeekIId);
					_wheelSeekIId = setInterval(_wheelSeekStop, 350, this);
				}
				_wheelSeekTime = Math.max(0, Math.min(display.duration ? display.duration : Infinity, _wheelSeekTime - p_delta * mouseWheelSeekStep * 1));
				display.time = _wheelSeekTime;
			}
		}
		updateAfterEvent();
		
	}

}