﻿/*

The MIT License

Copyright (c) 2007 Elvis Mehmedović

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

http://emehmedovic.com/

*/

// parameters supported: time, transition, onComplete, onUpdate

class xm.util.miniTweener {
	
	static var tweenerMcName:String = "__mini_tweener";
	
	static function addTween(p_mc:MovieClip, p_params:Object):Void {
		
		var tweenMc:MovieClip;
		if (!(tweenMc = p_mc[tweenerMcName])) {
			tweenMc = p_mc.createEmptyMovieClip(tweenerMcName, p_mc.getNextHighestDepth());
		}
		
		var time = p_params.time;
		var transition = typeof p_params.transition == "function" ? p_params.transition : (p_params.transition == "linear" ? easeNone : easeOutExpo);
		var startTime = getTimer();
		var onComplete = p_params.onComplete;
		var onUpdate = p_params.onUpdate;
		var rounded = p_params.rounded;
		
		delete p_params.time;
		delete p_params.transition;
		delete p_params.startTime;
		delete p_params.onComplete;
		delete p_params.onUpdate;
		delete p_params.rounded;
		
		for (var s:String in p_params) {
			
			var propMc:MovieClip;
			propMc.startTime = startTime;
			
			if (!(propMc = tweenMc["prop_"+s])) {
				propMc = tweenMc.createEmptyMovieClip("prop_"+s, tweenMc.getNextHighestDepth());
			}
			else {
				if (!propMc.completed) {
					propMc.onEnterFrame();
				}
				else {
					propMc.completed = false;
				}
			}
			
			propMc.time = time;
			propMc.transition = transition;
			propMc.startTime = startTime;
			propMc.prop = s;
			propMc.endPropValue = p_params[s];
			propMc.startPropValue = p_mc[s];
			propMc.onUpdate = onUpdate;
			propMc.onComplete = onComplete;
			propMc.rounded = rounded;
			
			propMc.onEnterFrame = function() {
			
				var p:MovieClip = this._parent._parent;
				var currentTime:Number = getTimer();
		
				var v = this.transition(Math.min(this.time * 1000, currentTime - this.startTime), this.startPropValue, this.endPropValue - this.startPropValue, this.time * 1000);
				p[this.prop] = this.rounded ? Math.round(v) : v;
				
				this.onUpdate.apply(p);
				
				if (currentTime - this.startTime >= this.time * 1000) {
					this.completed = true;
					this.onComplete.apply(p);
					if (currentTime - this.startTime >= this.time * 1000) {
						this.removeMovieClip();
					}
				}
			
			}
			
		}
		
	}
	
	static function removeTweens(p_mc:MovieClip) {
		p_mc[tweenerMcName].removeMovieClip();
	}
	
	static function easeOutExpo (t:Number, b:Number, c:Number, d:Number):Number {
		return (t==d) ? b+c : c * 1.001 * (-Math.pow(2, -10 * t/d) + 1) + b;
	}

	static function easeNone (t:Number, b:Number, c:Number, d:Number):Number {
		return c*t/d + b;
	}
	
}